@@ -102,7 +102,7 @@ while (argv[1][0] == '-')
 		}
 	else if (!strcmp(argv[0], "--version"))
 		{
-		fputs("Dap 2.1\n", stderr);
+		fputs("Dap 2.2\n", stderr);
 		exit(1);
 		}
 	else
@@ -136,13 +136,13 @@ char *argstr;	/* original argument list, as string */
 int argstrlen;	/* length of argument list, as string */
 
 if (!(arg = (char **) malloc(sizeof(char *) *
-		(argc + 4 + parseopts(edopts, NULL)))))
+		(argc + 5 + parseopts(edopts, NULL)))))
 	{	/* + 5 for .log, .lst, .c, --eval, (shell-command "dapruns ... &") */
 	perror("dap");
 	exit(1);
 	}
 a = 0;
-arg[a++] = editor;
+arg[a++] = argcpy(argv[1], 0);
 a += parseopts(edopts, arg + a);
 arg[a] = argcpy(argv[1], 4);
 suffix(arg[a++], ".log");
