@@ -23,7 +23,7 @@
 
 extern char *dap_dapname;	/* name of program running */
 extern DFILE *dap_in[2];	/* input dataset files */
-extern DFILE *dap_out;		/* output dataset file */
+extern DFILE *dap_out[2];	/* output dataset files */
 extern dataobs dap_obs[];	/* values from current input line */
 extern dataobs dap_prev[];	/* values from previous input line */
 extern int dap_ono;            /* specifies which set of variable values (dataobs) to use */
@@ -205,8 +205,8 @@ if (fname[0] == '<')
 			{
 			for (c = line[l]; c < dap_in[0]->dfile_ram->rfile_end && *c != '\n';
 						c++)
-				dap_putc(*c, dap_out);
-			dap_putc('\n', dap_out);
+				dap_putc(*c, dap_out[0]);
+			dap_putc('\n', dap_out[0]);
 			lastun = l;
 			}
 		else
@@ -216,8 +216,8 @@ if (fname[0] == '<')
 				;
 			}
 		}
-	flen = dap_out->dfile_ram->rfile_end - dap_out->dfile_ram->rfile_str;
-	memcpy(dap_in[0]->dfile_ram->rfile_str, dap_out->dfile_ram->rfile_str, flen);
+	flen = dap_out[0]->dfile_ram->rfile_end - dap_out[0]->dfile_ram->rfile_str;
+	memcpy(dap_in[0]->dfile_ram->rfile_str, dap_out[0]->dfile_ram->rfile_str, flen);
 	dap_in[0]->dfile_ram->rfile_end = dap_in[0]->dfile_ram->rfile_str + flen;
 	}
 else
@@ -272,10 +272,10 @@ else
 	strcat(command, " >> ");
 	strcat(command, dsrt);
 	system(command);
-	dap_free(dsrt);
-	dap_free(dfile);
+	dap_free(dsrt, "");
+	dap_free(dfile, "");
 	}
-dap_free(dsrt0);
+dap_free(dsrt0, "");
 }
 
 static void printhead(char **formstr, char *fname, int *varv, int nvar)
@@ -331,7 +331,7 @@ for (v = 0; v < nvar; v++)
 putc('\n', dap_lst);
 if (!wastitle)
 	title(NULL);
-dap_free(ttext);
+dap_free(ttext, "");
 }
 
 void print(char fname[], char *varlist)
@@ -406,9 +406,9 @@ for (obn = 1; step(); )
         obn++;
         }
 fflush(dap_lst);
-dap_free(varv);
-dap_free(formmem);
-dap_free(formstr);
+dap_free(varv, "");
+dap_free(formmem, "");
+dap_free(formstr, "");
 }
 
 int dap_mnsparse(char *varlist, char *outlist, int *varv, int *wtvar, int stats[])
@@ -519,8 +519,8 @@ for (outlist[0] = '\0'; i >= 0; )
 	for (i = j - 1; i >= 0 && tmplist[i] == ' '; --i)
 		;
 	}
-dap_free(vname);
-dap_free(tmplist);
+dap_free(vname, "");
+dap_free(tmplist, "");
 if (nonly)
 	return -nvar;
 return nvar;
@@ -644,7 +644,7 @@ for (pctptn = 0; statlist[s]; )
 	while (statlist[s] == ' ')
 		s++;
 	}
-dap_free(stat);
+dap_free(stat, "");
 }
 
 static void meansout(int varv[], int nvar, int nobs[], double sum[], double sumwt[],
@@ -875,7 +875,7 @@ if (stats[STEP])
 					min[v] + range[v] * fract;
 			output();
 			}
-		dap_free(range);
+		dap_free(range, "");
 		}
 	else
 		{
@@ -914,7 +914,7 @@ if (stats[TPROB])
 	output();
 	}
 dap_swap();
-dap_free(dn);
+dap_free(dn, "");
 }
 
 int dap_list(char *varlist, int *varv, int maxvars)
@@ -963,7 +963,7 @@ for (nvars = 0; varlist[m]; )
 	while (varlist[m] == ' ')
 		m++;
 	}
-dap_free(mname);
+dap_free(mname, "");
 return nvars;
 }
 
@@ -1122,18 +1122,18 @@ for (more = 1; more; )
 			nnan[v]++;
 		}
 	}
-dap_free(outname);
-dap_free(varv);
-dap_free(markv);
-dap_free(nobs);
-dap_free(outlist);
-dap_free(wtvar);
-dap_free(sum);
-dap_free(sumwt);
-dap_free(ss);
-dap_free(min);
-dap_free(max);
-dap_free(nnan);
+dap_free(outname, "");
+dap_free(varv, "");
+dap_free(markv, "");
+dap_free(nobs, "");
+dap_free(outlist, "");
+dap_free(wtvar, "");
+dap_free(sum, "");
+dap_free(sumwt, "");
+dap_free(ss, "");
+dap_free(min, "");
+dap_free(max, "");
+dap_free(nnan, "");
 }
 
 typedef struct
@@ -1639,7 +1639,7 @@ if (ncolvar < 2)
 	colvar[0] = -1;
 	ncolvar = 2;
 	}
-dap_free(vname);
+dap_free(vname, "");
 }
 
 void table(char *fname, char *rowvars, char *colvars, char *format, char *marks)
@@ -1720,7 +1720,7 @@ for (r = 0; r < nrowvar; r++)
 	if (dap_obs[0].do_len[rowvar[r]] <= 0)
 		{
 		if (dap_obs[0].do_str[rowvar[r]])
-			dap_free(dap_obs[0].do_str[rowvar[r]]);
+			dap_free(dap_obs[0].do_str[rowvar[r]], "");
 		dap_obs[0].do_str[rowvar[r]] = dap_malloc(rtitlesp + 1, "");
 		}
 	}
@@ -1729,7 +1729,7 @@ for (c = 0; c < ncolvar - 1; c++)
 	if (dap_obs[0].do_len[colvar[c]] <= 0)
 		{
 		if (dap_obs[0].do_str[colvar[c]])
-			dap_free(dap_obs[0].do_str[colvar[c]]);
+			dap_free(dap_obs[0].do_str[colvar[c]], "");
 		dap_obs[0].do_str[colvar[c]] = dap_malloc(rtitlesp + 1, "");
 		}
 	}
@@ -1923,12 +1923,12 @@ while (step())
 		output();	/* and write line to output dataset */
 		}
 	}
-dap_free(classvar);
-dap_free(valuevar);
-dap_free(var);
-dap_free(skiplist);
-dap_free(outname);
-dap_free(varname);
+dap_free(classvar, "");
+dap_free(valuevar, "");
+dap_free(var, "");
+dap_free(skiplist, "");
+dap_free(outname, "");
+dap_free(varname, "");
 }
 
 void join(char *fname, char *partvars, char *valuevar)
@@ -2074,11 +2074,11 @@ for (more = 1, nv = 0; more; nv++)
 				dap_obs[dap_ono].do_str[valv]);
 		}
 	}
-dap_free(outname);
-dap_free(newv);
-dap_free(partvars1);
-dap_free(classvar);
-dap_free(skiplist);
-dap_free(varspec);
-dap_free(partv);
+dap_free(outname, "");
+dap_free(newv, "");
+dap_free(partvars1, "");
+dap_free(classvar, "");
+dap_free(skiplist, "");
+dap_free(varspec, "");
+dap_free(partv, "");
 }
