@@ -200,10 +200,10 @@ if (stats[QRANGE])
 	output();
 	}
 dap_swap();
-dap_free(pctptmem);
-dap_free(pctpt);
-dap_free(excessmem);
-dap_free(excess);
+dap_free(pctptmem, "");
+dap_free(pctpt, "");
+dap_free(excessmem, "");
+dap_free(excess, "");
 }
 
 static void pctile1(double ***val, int nobs, int nvar, int *varv, int *stats)
@@ -414,14 +414,14 @@ for (nobs = 0, more = 1; more; nobs++)
 			}
 		}
 	}
-dap_free(outname);
-dap_free(varv);
-dap_free(markv);
-dap_free(wtvar);
-dap_free(outlist);
-dap_free(valmem);
-dap_free(valpair);
-dap_free(val);
+dap_free(outname, "");
+dap_free(varv, "");
+dap_free(markv, "");
+dap_free(wtvar, "");
+dap_free(outlist, "");
+dap_free(valmem, "");
+dap_free(valpair, "");
+dap_free(val, "");
 }
 
 static void corr1(int *varv, int nvar, double **cormat, double ss[], int nobs)
@@ -605,14 +605,14 @@ for (nobs = 0, more = 1; more; nobs++)
 			}
 		}
 	}
-dap_free(markv);
-dap_free(varv);
-dap_free(outname);
-dap_free(outlist);
-dap_free(cormem);
-dap_free(cormat);
-dap_free(sum);
-dap_free(ss);
+dap_free(markv, "");
+dap_free(varv, "");
+dap_free(outname, "");
+dap_free(outlist, "");
+dap_free(cormem, "");
+dap_free(cormat, "");
+dap_free(sum, "");
+dap_free(ss, "");
 }
 
 #define GRPNUMBER (-1)
@@ -755,7 +755,7 @@ for (nvar = 0, number = 0; varspec[s]; )
 	while (varspec[s] == ' ')
 		s++;
 	}
-dap_free(varname);
+dap_free(varname, "");
 if (number < 0)
 	return -4 * nvar + number;
 return nvar;
@@ -1009,17 +1009,17 @@ for (dap_mark(), nobs = 0, nnan = 0, more = 1; more; )
 	}
 if (!number)
 	{
-	dap_free(nummem);
-	dap_free(numval);
-	dap_free(ptmem);
-	dap_free(point);
+	dap_free(nummem, "");
+	dap_free(numval, "");
+	dap_free(ptmem, "");
+	dap_free(point, "");
 	}
-dap_free(outname);
-dap_free(grpname);
-dap_free(grpv);
-dap_free(markv);
-dap_free(varv);
-dap_free(ctype);
+dap_free(outname, "");
+dap_free(grpname, "");
+dap_free(grpv, "");
+dap_free(markv, "");
+dap_free(varv, "");
+dap_free(ctype, "");
 }
 
 #define NFREQSTAT 13
@@ -1122,7 +1122,7 @@ while (stats[s])
 	while (stats[s] == ' ')
 		s++;
 	}
-dap_free(stat);
+dap_free(stat, "");
 }
 
 static int findlev(int v, char **level, int *nlevels)
@@ -1556,14 +1556,14 @@ if (statv[FREQNOM])
 		2.0 * uv / (hx + hy), var);
 	}
 dap_swap();
-dap_free(expmem);
-dap_free(expect);
-dap_free(rowsum);
-dap_free(colsum);
-dap_free(amem);
-dap_free(a);
-dap_free(dmem);
-dap_free(d);
+dap_free(expmem, "");
+dap_free(expect, "");
+dap_free(rowsum, "");
+dap_free(colsum, "");
+dap_free(amem, "");
+dap_free(a, "");
+dap_free(dmem, "");
+dap_free(d, "");
 }
 
 static int freqparse(char *varlist, int *varv, int *wt)
@@ -1625,7 +1625,7 @@ for (nvars = 0, wtvar = 0; varlist[m]; )
         while (varlist[m] == ' ')
                 m++;
         }
-dap_free(mname);
+dap_free(mname, "");
 return nvars;
 }
 
@@ -1803,15 +1803,15 @@ for (dap_mark(), sumcount = 0.0, more = 1; more; )
 	else
 		sumcount += 1.0;
 	}
-dap_free(outname);
-dap_free(outlist);
-dap_free(markv);
-dap_free(varv);
-dap_free(tabmem);
-dap_free(tab);
-dap_free(levmem);
-dap_free(level[0]);
-dap_free(level[1]);
+dap_free(outname, "");
+dap_free(outlist, "");
+dap_free(markv, "");
+dap_free(varv, "");
+dap_free(tabmem, "");
+dap_free(tab, "");
+dap_free(levmem, "");
+dap_free(level[0], "");
+dap_free(level[1], "");
 }
 
 static void trim1(double *vpct, int nvar, double **val,
@@ -1918,7 +1918,7 @@ for (nvar = 0; trimspec[n]; )
 	while (trimspec[n] == ' ')
 		n++;
 	}
-dap_free(varname);
+dap_free(varname, "");
 return nvar;
 }
 
@@ -2000,12 +2000,12 @@ for (dap_mark(), nobs = 0, more = 1; more; nobs++)
 		exit(1);
 		}
 	}
-dap_free(outname);
-dap_free(markv);
-dap_free(varv);
-dap_free(vpct);
-dap_free(valmem);
-dap_free(val);
-dap_free(vmin);
-dap_free(vmax);
+dap_free(outname, "");
+dap_free(markv, "");
+dap_free(varv, "");
+dap_free(vpct, "");
+dap_free(valmem, "");
+dap_free(val, "");
+dap_free(vmin, "");
+dap_free(vmax, "");
 }
