@@ -151,11 +151,11 @@ if (doaxes)
 dap_swap();
 if (doaxes)
 	{
-	dap_free(title0);
+	dap_free(title0, "");
 	title0 = NULL;
 	}
-dap_free(title1);
-dap_free(axspec);
+dap_free(title1, "");
+dap_free(axspec, "");
 }
 
 /* Parse variable names with optional axis renaming
@@ -365,12 +365,12 @@ for (nobs = 0, nnan = 0, pn = 0, more = 1; more; )
 		exit(1);
 		}
 	}
-dap_free(x);
-dap_free(y);
-dap_free(markv);
-dap_free(xyname);
-dap_free(xname);
-dap_free(yname);
+dap_free(x, "");
+dap_free(y, "");
+dap_free(markv, "");
+dap_free(xyname, "");
+dap_free(xname, "");
+dap_free(yname, "");
 return p;
 }
 
@@ -574,7 +574,7 @@ for (s = 0, small = 0; s < NSIMUL; s++)
 	if (w < w0)
 		small++;
 	}
-dap_free(y);
+dap_free(y, "");
 return ((double) small) / ((double) NSIMUL);
 }
 
@@ -644,7 +644,7 @@ if (6 < nobs && nobs <= 500)
 	w *= w / ss;
 	if ((prob = probw(swa, nobs, w)) < 0.001)
 		prob = 0.001;
-	dap_free(swa);
+	dap_free(swa, "");
 	if (p)	/* if q-q plot requested */
 		sprintf(caption, "q-q plot: W|0| = %.4f, P[W < W|0|] = %.3f", w, prob);
 	dap_head(markv, nmark);
@@ -814,11 +814,11 @@ for (nobs = 0, nnan = 0, pn = 0, more = 1; more; )
 		exit(1);
 		}
 	}
-dap_free(x);
-dap_free(y);
-dap_free(varname);
-dap_free(varlabel);
-dap_free(markv);
+dap_free(x, "");
+dap_free(y, "");
+dap_free(varname, "");
+dap_free(varlabel, "");
+dap_free(markv, "");
 return p;
 }
 
@@ -1112,10 +1112,10 @@ if (nmark)
 		}
 	}
 pict_autoaxes(p + pn, xname, htitle, axspec, xfunct, NULL, caption, PORTRAIT);
-dap_free(caption);
-dap_free(word);
-dap_free(h);
-dap_free(part);
+dap_free(caption, "");
+dap_free(word, "");
+dap_free(h, "");
+dap_free(part, "");
 }
 
 /* Display histogram */
@@ -1250,12 +1250,12 @@ for (nobs = 0, nnan = 0, pn = 0, more = 1; more; )
                 }
         }
 if (nvar == 1)
-	dap_free(x);
+	dap_free(x, "");
 else
-	dap_free(xw);
-dap_free(markv);
-dap_free(xwname);
-dap_free(xname);
+	dap_free(xw, "");
+dap_free(markv, "");
+dap_free(xwname, "");
+dap_free(xname, "");
 return p;
 }
 
@@ -1270,6 +1270,11 @@ char *ylist;	/* copy of ylist0 without axis renaming, if any */
 char *x1list;	/* copy of x1list0 without axis renaming, if any */
 int l;		/* index to strings */
 int varv[1];
+int typen;	/* index of type variable */
+int sortord[4];	/* sort order for types for plot: 0 = LOWER, 1 = OBS,
+		 * 2 = PRED, 3 = UPPER
+		 */
+int s;		/* index to sortord */
 char *mnsname;	/* name of dataset for output of means() */
 char *regname;	/* name of dataset for output of linreg() */
 char *srtarg;	/* arguments to sort() */
@@ -1289,6 +1294,32 @@ for (l = 0; x1list0[l] && x1list0[l] != '`'; l++)
 	x1list[l] = x1list0[l];
 x1list[l] = '\0';
 inset(fname);
+if ((typen = dap_varnum("_type_")) < 0)
+	{
+	fprintf(dap_err, "(plotlinreg) missing type variable in %s\n", fname);
+	exit(1);
+	}
+step();
+for (s = 0; s < 4; s++)
+	sortord[s] = s;
+if (strcmp(dap_obs[0].do_str[typen], "LOWER") < 0)
+	{	/* the 'OBS' value will appear first */
+	sortord[0] = 1;
+	sortord[1] = 0;
+	}
+else if (strcmp(dap_obs[0].do_str[typen], "PRED") < 0)
+	;
+else if (strcmp(dap_obs[0].do_str[typen], "UPPER") < 0)
+	{	/* the 'OBS' value will appear between 'PRED' and 'UPPER' */
+	sortord[1] = 2;
+	sortord[2] = 1;
+	}
+else
+	{
+	sortord[1] = 3;
+	sortord[2] = 1;
+	sortord[3] = 2;
+	}
 dap_list(ylist, varv, 1);	/* check that it's only 1 variable */
 dap_list(x1list, varv, 1);	/* check that it's only 1 variable */
 mnsname = dap_malloc(strlen(fname) + 5, "");
@@ -1321,21 +1352,21 @@ strcat(plotstyle, style);
 p = plot(srtname, plotvars, plotmarks, plotstyle, NULL, NULL, 4 * nmarks);
 for (pn = 0; pn < nmarks; pn++)
 	{
-	strcpy(p[4 * pn + 0].pict_type, "LINE");
-	strcpy(p[4 * pn + 2].pict_type, "LINE");
-	strcpy(p[4 * pn + 3].pict_type, "LINE");
-	p[4 * pn + 0].pict_dash = 4.0;
-	p[4 * pn + 3].pict_dash = 4.0;
+	strcpy(p[4 * pn + sortord[0]].pict_type, "LINE");
+	strcpy(p[4 * pn + sortord[2]].pict_type, "LINE");
+	strcpy(p[4 * pn + sortord[3]].pict_type, "LINE");
+	p[4 * pn + sortord[0]].pict_dash = 4.0;
+	p[4 * pn + sortord[3]].pict_dash = 4.0;
 	}
-dap_free(ylist);
-dap_free(x1list);
-dap_free(mnsname);
-dap_free(regname);
-dap_free(srtarg);
-dap_free(srtname);
-dap_free(plotvars);
-dap_free(plotmarks);
-dap_free(plotstyle);
+dap_free(ylist, "");
+dap_free(x1list, "");
+dap_free(mnsname, "");
+dap_free(regname, "");
+dap_free(srtarg, "");
+dap_free(srtname, "");
+dap_free(plotvars, "");
+dap_free(plotmarks, "");
+dap_free(plotstyle, "");
 return p;
 }
 
@@ -1499,22 +1530,22 @@ for (pn = 0; pn < nmarks; pn ++)
 	p[4 * pn + 0].pict_dash = 4.0;
 	p[4 * pn + 3].pict_dash = 4.0;
 	}
-dap_free(yspec);
-dap_free(x1list);
-dap_free(trlname);
-dap_free(grpname);
-dap_free(grparg);
-dap_free(grpvar);
-dap_free(mnsarg);
-dap_free(mnsname);
-dap_free(lgrname);
-dap_free(srtarg);
-dap_free(srtname);
-dap_free(plotvars);
-dap_free(plotmarks);
-dap_free(casevar);
-dap_free(casevar0);
-dap_free(plotstyle);
+dap_free(yspec, "");
+dap_free(x1list, "");
+dap_free(trlname, "");
+dap_free(grpname, "");
+dap_free(grparg, "");
+dap_free(grpvar, "");
+dap_free(mnsarg, "");
+dap_free(mnsname, "");
+dap_free(lgrname, "");
+dap_free(srtarg, "");
+dap_free(srtname, "");
+dap_free(plotvars, "");
+dap_free(plotmarks, "");
+dap_free(casevar, "");
+dap_free(casevar0, "");
+dap_free(plotstyle, "");
 return p;
 }
 
@@ -1677,16 +1708,16 @@ for (pn = 0; pn < nparts; pn++)
 	strcpy(p[2 * pn].pict_type, "IBEA");
 if (partvars && partvars[0])
 	{
-	dap_free(partv);
-	dap_free(mnslist);
-	dap_free(plotparts);
+	dap_free(partv, "");
+	dap_free(mnslist, "");
+	dap_free(plotparts, "");
 	}
-dap_free(mnsname);
-dap_free(errname);
-dap_free(srtname);
-dap_free(ebar);
-dap_free(overstr);
-dap_free(srtarg);
-dap_free(plotvars);
+dap_free(mnsname, "");
+dap_free(errname, "");
+dap_free(srtname, "");
+dap_free(ebar, "");
+dap_free(overstr, "");
+dap_free(srtarg, "");
+dap_free(plotvars, "");
 return p;
 }
