@@ -231,8 +231,8 @@ if (nonz)
 			}
 		}
 	}
-dap_free(misscol);
-dap_free(term);
+dap_free(misscol, "");
+dap_free(term, "");
 if (elimrow == ncols)
 	return elimrow;
 fputs("error terms insufficient to impute missing cells\n", dap_err);
@@ -405,7 +405,7 @@ for (t = 0; (rterm && t < nterms) || t < 1; t++)
 		cr += updown;
 		}
 	}
-dap_free(term);
+dap_free(term, "");
 return df;
 }
 
@@ -431,11 +431,12 @@ for (r = 0; r < nerrors + ncontrasts; r++)
 		}
 	if (fabs(rowsum) > dap_zerotol * rowmax)
 		{
+		fprintf(dap_err, "(sumcheck:%s) Unable to fit model:\n", caller);
 		if (r < nerrors)
-			fprintf(dap_err, "(sumcheck:%s) Error %d sums to %g:\n",
+			fprintf(dap_err, "Error %d sums to %g:\n",
 						caller, r, rowsum);
 		else
-			fprintf(dap_err, "(sumcheck:%s) Contrast %d sums to %g:\n",
+			fprintf(dap_err, "Contrast %d sums to %g:\n",
 					caller, r - nerrors, rowsum);
 		fprintf(dap_err, "%s%d (%x) ",
 				((r < nerrors) ? "E" : "C"),
@@ -527,7 +528,7 @@ for (firstv = 1; test[t]; )
 		firstv = 1;
 		}
 	}
-dap_free(vname);
+dap_free(vname, "");
 return nterms;
 }
 
@@ -725,12 +726,12 @@ for (t = 1, factor = 0; t <= nterm; t++)
 		factor++;
 		}
 	}
-dap_free(cum);
-dap_free(effrow);
-dap_free(factlev);
-dap_free(emsc);
-dap_free(lensq);
-dap_free(used);
+dap_free(cum, "");
+dap_free(effrow, "");
+dap_free(factlev, "");
+dap_free(emsc, "");
+dap_free(lensq, "");
+dap_free(used, "");
 return factor;
 }
 
@@ -1057,11 +1058,11 @@ dap_obs[0].do_dbl[varv[0]] = dedfi;
 output();
 if (nden)
 	{
-	dap_free(emsmem);
-	dap_free(emscoeff);
+	dap_free(emsmem, "");
+	dap_free(emscoeff, "");
 	}
-dap_free(indep);
-dap_free(termv);
+dap_free(indep, "");
+dap_free(termv, "");
 }
 
 static void puttest(char *testv, int ntest, int *varv, int nvars)
@@ -1394,23 +1395,23 @@ for (ncells = 0, ncontrasts = 0, nerrors = 0,
 		more = step();
 		}
 	}
-dap_free(comem);
-dap_free(coeff);
-dap_free(tstname);
-dap_free(varv);
-dap_free(markv);
-dap_free(rterm);
-dap_free(numv);
-dap_free(denv);
-dap_free(levmem);
-dap_free(levptr);
-dap_free(levval);
-dap_free(nlevels);
-dap_free(levelmem);
-dap_free(level);
-dap_free(mean);
-dap_free(nobs);
-dap_free(vari);
+dap_free(comem, "");
+dap_free(coeff, "");
+dap_free(tstname, "");
+dap_free(varv, "");
+dap_free(markv, "");
+dap_free(rterm, "");
+dap_free(numv, "");
+dap_free(denv, "");
+dap_free(levmem, "");
+dap_free(levptr, "");
+dap_free(levval, "");
+dap_free(nlevels, "");
+dap_free(levelmem, "");
+dap_free(level, "");
+dap_free(mean, "");
+dap_free(nobs, "");
+dap_free(vari, "");
 }
 
 static void putlev(int *nlevels, int *varv, int nvars, char ***levval)
@@ -1569,10 +1570,10 @@ for (tt = 0, r = 0; tt < 3; tt++)
 			}
 		}
 	}
-dap_free(nest);
-dap_free(iv);
-dap_free(rlevel);
-dap_free(change);
+dap_free(nest, "");
+dap_free(iv, "");
+dap_free(rlevel, "");
+dap_free(change, "");
 }
 
 static int cmpstr(char **s1, char **s2)
@@ -1818,17 +1819,17 @@ for (celln = 0; celln < ncells; celln++)
 			break;
 		}
 	}
-dap_free(comem);
-dap_free(coeff);
-dap_free(level);
-dap_free(clevmem);
-dap_free(clevel);
-dap_free(nobs);
-dap_free(mean);
-dap_free(vari);
-dap_free(rterm);
-dap_free(indep);
-dap_free(nonz);
+dap_free(comem, "");
+dap_free(coeff, "");
+dap_free(level, "");
+dap_free(clevmem, "");
+dap_free(clevel, "");
+dap_free(nobs, "");
+dap_free(mean, "");
+dap_free(vari, "");
+dap_free(rterm, "");
+dap_free(indep, "");
+dap_free(nonz, "");
 }
 
 void effects(char *fname, char *varlist, char *model, char *marks)
@@ -1955,21 +1956,22 @@ for (more = 1, incells = 0; more; incells++)
 				fprintf(dap_err, "%s (%s) ",
 					dap_obs[0].do_nam[varv[v]], dap_obs[0].do_str[varv[v]]);
 				}
+			putc('\n', dap_err);
 			exit(1);
 			}
 		}
 	}
 if (model && model[0])
 	ftest(conname, varlist, "", "", marks);
-dap_free(conname);
-dap_free(outlist);
-dap_free(varv);
-dap_free(markv);
-dap_free(termv);
-dap_free(levval);
-dap_free(nlevels);
-dap_free(levmem);
-dap_free(levptr);
+dap_free(conname, "");
+dap_free(outlist, "");
+dap_free(varv, "");
+dap_free(markv, "");
+dap_free(termv, "");
+dap_free(levval, "");
+dap_free(nlevels, "");
+dap_free(levmem, "");
+dap_free(levptr, "");
 }
 
 /* Compute LS means, differences, and probabilities for treatment comparisons */
@@ -2272,13 +2274,13 @@ case DUNNETT:
 		alpha, pt);
 	break;
 	}
-dap_free(indep);
-dap_free(lsmean);
-dap_free(effinvn);
-dap_free(diffmem);
-dap_free(diff);
-dap_free(probmem);
-dap_free(prob);
+dap_free(indep, "");
+dap_free(lsmean, "");
+dap_free(effinvn, "");
+dap_free(diffmem, "");
+dap_free(diff, "");
+dap_free(probmem, "");
+dap_free(prob, "");
 dap_swap();
 }
 
@@ -2540,19 +2542,19 @@ while (*format == ' ')	/* here we want to deny user sorting */
 if (*format == 's')
 	format++;
 table(lsmsrt, args1, args2, format, marks);
-dap_free(varv);
-dap_free(markv);
-dap_free(lsmname);
-dap_free(lsmsrt);
-dap_free(varstr);
-dap_free(treat2);
-dap_free(args1);
-dap_free(args2);
-dap_free(levmem);
-dap_free(levval);
-dap_free(comem);
-dap_free(coeff);
-dap_free(mean);
-dap_free(nobs);
-dap_free(vari);
+dap_free(varv, "");
+dap_free(markv, "");
+dap_free(lsmname, "");
+dap_free(lsmsrt, "");
+dap_free(varstr, "");
+dap_free(treat2, "");
+dap_free(args1, "");
+dap_free(args2, "");
+dap_free(levmem, "");
+dap_free(levval, "");
+dap_free(comem, "");
+dap_free(coeff, "");
+dap_free(mean, "");
+dap_free(nobs, "");
+dap_free(vari, "");
 }
