@@ -45,30 +45,38 @@ for (diff = 0, i = 0; !diff && i < nmark; i++)
 return !diff;
 }
 
+/* xymat: 0 row contains sum of values for x and y vars except for _intercept_
+ * (1 to nx-1) x (1 to nx-1) contains SS for x-variables;
+ * (1 to nx-1) x (nx to nvar) contains SS for x-vars with each y-var
+ * (nx to nvar-1) x (1 to nx-1) contains SS for x-vars with each y-var
+ * (nx, nx) to (nvar-1,nvar-1) diagonal contains SS for each y-var
+ */
 void linreg1(double **xymat, int *varv,
 		int nx0, int nx, int ny, int nobs, int *xvarv,
-		int *markv, int *xmarkv, int nmark, double level)
+		int *markv, int *xmarkv, int nmark, double level,
+		int respn, int param1n, int param2n, int covn, int partv[])
 {
-double *invmem;
-double **inv;
-int r, c;
-int rr, cc;
-double pivot;
+double *invmem;			/* memory allocated for inverse of SS matrix */
+double **inv;			/* pointers for inverse matrix */
+int r, c;			/* row and column */
+int rr, cc;			/* row and column */
+double pivot;			/* numerical value of pivot element */
 double tmp, tmp2;
-int typen;
-double dnobs;
-double *rss0, *rss1, *rss;
-double *f, *fch;
-int df;
-double ddf;
-double xi, xj;
-static double tpt;
-int yn;
+int typen;			/* index of _type_ variable */
+double dnobs;			/* number of observations, as double */
+double *rss0, *rss1, *rss;	/* reduced sums of squares for y vs x0s, y vs x1s, y vs y */
+double *f, *fch;		/* F and F change values */
+int df;				/* degrees of freedom */
+double ddf;			/* degrees of freedom, as double */
+double xi, xj;			/* indexes to x-variables */
+static double tpt;		/* point in t-distribution */
+int yn;				/* index to y variables */
 int i, j;
-double *pred;
-double *sepred;
+double *pred;			/* array of predicted values */
+double *sepred;			/* array of SEs for predictions */
+int v;				/* index to variables */
 
-dap_swap();
+dap_swap();	/* it's cleaner to have linreg1 get the previous part's mark variables */
 if ((typen = dap_varnum("_type_")) < 0)
 	{
 	fprintf(dap_err, "(linreg1) Missing _type_ variable.\n");
@@ -86,8 +94,10 @@ fch = (double *) dap_malloc(sizeof(double) * ny, "");
 pred = (double *) dap_malloc(sizeof(double) * ny, "");
 sepred = (double *) dap_malloc(sizeof(double) * ny, "");
 dnobs = (double) nobs;
+/* row 0 of xymat will now be the means of the observations */
 for (c = 1; c < nx + ny; c++)
 	xymat[0][c] /= dnobs;
+/* initialize the inverse to the identity before performing row ops */
 for (r = 1; r < nx; r++)
 	for (c = 1; c < nx; c++)
 		{
@@ -96,36 +106,39 @@ for (r = 1; r < nx; r++)
 		else
 			inv[r][c] = 0.0;
 		}
+/* now perform row ops for the x-variables, from row 1 to nx-1 */
 for (c = 1; c < nx; c++)
 	{
-	if (c == 1)
+	/* these could have been done before, but can be done here, too */
+	if (c == 1)	/* initialize RSS for x0 vars to SSs of the y's */
 		{
 		for (cc = 0; cc < ny; cc++)
 			rss0[cc] = xymat[nx + cc][nx + cc];
 		}
-	if (c == nx0)
+	if (c == nx0)	/* init RSS for x1 vars to SSs of the y's after reduction of x0 rows */
 		{
 		for (cc = 0; cc < ny; cc++)
 			rss1[cc] = xymat[nx + cc][nx + cc];
 		}
-	pivot = xymat[c][c];
-	if (pivot != 0.0)
-		{
+	pivot = xymat[c][c];	/* default pivot */
+	if (pivot != 0.0)	/* OK, trusting the SS matrix not to be weird */
+		{		/* should not require pivoting */
 		for (rr = c + 1; rr < nx + ny; rr++)
 			{
-			tmp = xymat[rr][c] / pivot;
-			xymat[rr][c] = 0.0;
+			tmp = xymat[rr][c] / pivot;	/* mult factor for row op */
+			xymat[rr][c] = 0.0;		/* eliminate below (c,c) elt */
 			for (cc = c + 1; cc < nx + ny; cc++)
 				{
 				if (rr < nx || cc < nx || rr == cc)
 					{
 					xymat[rr][cc] -= tmp * xymat[c][cc];
+					/* zero out entries below tolerance */
 					if (fabs(xymat[rr][cc]) < dap_tol * pivot)
 						xymat[rr][cc] = 0.0;
 					}
 				}
 			if (rr < nx)
-				{
+				{	/* repeat row reduction to get inverse */
 				for (cc = 1; cc < nx; cc++)
 					{
 					inv[rr][cc] -= tmp * inv[c][cc];
@@ -141,6 +154,7 @@ for (c = 1; c < nx; c++)
 		exit(1);
 		}
 	}
+/* now need to do upward row ops to get inverse */
 for (c = nx - 1; c > 0; --c)
 	{
 	for (r = c - 1; r > 0; --r)
@@ -158,15 +172,20 @@ for (c = nx - 1; c > 0; --c)
 	}
 df = nobs - nx;
 ddf = (double) df;
+/* RSS for y-vars are the diagonal entries of xymat
+ * and F stats are ratios of differences of SSs to RSS for the y-vars
+ */
 for (c = 0; c < ny; c++)
 	{
-	rss[c] = xymat[nx + c][nx + c];
+	rss[c] = xymat[nx + c][nx + c];		/* SSs of the y-vars after reduction */
 	f[c] = (rss0[c] - rss[c]) / rss[c] * ddf / ((double) nx - 1);
 	fch[c] = (rss1[c] - rss[c]) / rss[c] * ddf / ((double) nx - nx0);
 	}
+/* now copy inverse of SS of the x-vars back to xymat */
 for (r = 1; r < nx; r++)
 	for (c = 1; c < nx; c++)
 		xymat[r][c] = inv[r][c];
+/* column 0 will be the minus cross-product of inv SS with the means */
 for (r = 1; r < nx; r++)
 	for (c = 1; c < nx; c++)
 		xymat[r][0] -= xymat[r][c] * xymat[0][c];
@@ -178,6 +197,19 @@ for (c = 0; c < ny; c++)
 		xymat[0][nx + c] -= xymat[0][cc] * xymat[cc][nx + c];
 for (c = 1; c < nx; c++)
 	xymat[0][c] = xymat[c][0];
+/* now print and output estimates and SEs, later output the covariance matrix */
+dap_ono = 2;
+/* set up partvars for covariance matrix dataset */
+for (v = 0; v < nmark; v++)
+        {
+        if (dap_obs[0].do_len[markv[v]] == DBL)
+                dap_obs[dap_ono].do_dbl[partv[v]] = dap_obs[0].do_dbl[markv[v]];
+        else if (dap_obs[0].do_len[markv[v]] == INT)
+                dap_obs[dap_ono].do_int[partv[v]] = dap_obs[0].do_int[markv[v]];
+        else
+                strcpy(dap_obs[dap_ono].do_str[partv[v]],
+                        dap_obs[0].do_str[markv[v]]);
+        }
 fprintf(dap_lst, "Reduced | full model regressors:");
 for (r = 0; r < nx0; r++)
 	fprintf(dap_lst, " %s", dap_obs[0].do_nam[varv[r]]);
@@ -204,15 +236,47 @@ for (c = 0; c < ny; c++)
 		nobs - nx);
 	for (r = 0; r < nx; r++)
 		{
+/* tmp = sqrt(RSS for y with itself after reduction / deg freedom * inv SS for x-var)
+ * this is the standard error, the denominator for the t-value
+ */
 		tmp = sqrt(rss[c] / ddf * xymat[r][r]);
+/* tmp2 is the t-value because xymat[r][nx + c] is the estimate */
 		tmp2 = xymat[r][nx + c] / tmp;
 		fprintf(dap_lst, "   %-15s %12.6g %12.6g %12.6g  %14.5f\n",
 			dap_obs[0].do_nam[varv[r]],
 			xymat[r][nx + c], tmp,
 			tmp2,
 			0.00001 * ceil(200000.0 * probt(fabs(tmp2), nobs - nx)));
+                strcpy(dap_obs[dap_ono].do_str[typen], "ESTIMATE");
+                strcpy(dap_obs[dap_ono].do_str[respn], dap_obs[0].do_nam[varv[nx + c]]);
+                strcpy(dap_obs[dap_ono].do_str[param2n], dap_obs[0].do_nam[varv[r]]);
+                strcpy(dap_obs[dap_ono].do_str[param1n], "");
+                dap_obs[dap_ono].do_dbl[covn] = xymat[r][nx + c];
+                output();
 		}
 	}
+/* now we write out the covariance matrix */
+strcpy(dap_obs[dap_ono].do_str[typen], "COVAR");
+for (yn = 0; yn < ny; yn++)
+	{
+	strcpy(dap_obs[dap_ono].do_str[respn], dap_obs[0].do_nam[varv[nx + yn]]);
+	tmp = rss[yn] / ddf;
+	/* "rows" are indexed by r */
+	for (r = 0; r < nx; r++)
+		{
+		strcpy(dap_obs[dap_ono].do_str[param1n], dap_obs[0].do_nam[varv[r]]);
+		/* "columns" are indexed by c */
+		for (c = 0; c < nx; c++)
+			{
+			strcpy(dap_obs[dap_ono].do_str[param2n], dap_obs[0].do_nam[varv[c]]);
+			dap_obs[dap_ono].do_dbl[covn] = tmp * xymat[r][c];
+			output();
+			}
+		}
+	}
+/* now back to .reg output */
+dap_ono = 0;
+/* now we write out the OBS, PRED, LOWER, and UPPER values for the xname values */
 if (level < 1.0)
 	tpt = tpoint(0.5 * (1.0 - level), nobs - nx);
 else
@@ -226,8 +290,8 @@ if (xvarv[0] < 0)
 while (matchmark(markv, xmarkv, nmark, level))
 	{
 	dap_ono = 0;
-	if (xvarv[0] >= 0)
-		{
+	if (xvarv[0] >= 0)	/* if dataset for x-values specified */
+		{		/* there are no observed values */
 		for (i = 1; i < nx; i++)
 			dap_obs[0].do_dbl[varv[i]] = dap_obs[1].do_dbl[xvarv[i - 1]];
 		}
@@ -274,38 +338,45 @@ while (matchmark(markv, xmarkv, nmark, level))
 dap_ono = 0;
 if (xvarv[0] >= 0)
 	dap_swap();
-dap_free(invmem);
-dap_free(inv);
-dap_free(rss0);
-dap_free(rss1);
-dap_free(rss);
-dap_free(f);
-dap_free(fch);
-dap_free(pred);
-dap_free(sepred);
+dap_free(invmem, "");
+dap_free(inv, "");
+dap_free(rss0, "");
+dap_free(rss1, "");
+dap_free(rss, "");
+dap_free(f, "");
+dap_free(fch, "");
+dap_free(pred, "");
+dap_free(sepred, "");
 }
 
+/* xname is dataset of x-values at which to write out OBS,PRED,LOWER, and UPPER values */
 void linreg(char *fname, char *ylist, char *x0list, char *x1list,
 				char *marks, char *xname, double level)
 {
-char *regname;
-int *varv;
-int *xvarv;
-int ny;
-int nx0, nx1;
-int nx;
-int nvar;
-int nxx;
-int *markv;
-int *xmarkv;
-int nmark;
-int v, w;
+char *regname;	/* name of .reg dataset for writing out OBS,PRED,LOWER, and UPPER values */
+int *varv;	/* indexes of all variables */
+int *xvarv;	/* indexes of x-variables in dataset xname */
+int ny;		/* number of y-variables */
+int nx0, nx1;	/* num of x-vars in reduced (includes _intercept_), full-reduced models */
+int nx;		/* number of x-variables: nx0 + nx1 */
+int nvar;	/* number of x- and y-variables: nx + ny */
+int nxx;	/* number of x-variables not including _intercept_ */
+int *markv;	/* indexes of mark variables for partitioning the dataset */
+int *xmarkv;	/* if xname is given, indexes of mark variables in dataset xname */
+int nmark;	/* number of mark variables (must be equal for fname and xname datasets) */
+int v, w;	/* indexes to index vectors */
 double tmp;
-int nobs;
-double dnobs;
-double *xymem;
-double **xymat;
-int more;
+int nobs;	/* number of observations in the dataset fname */
+double dnobs;	/* nobs converted to double */
+double *xymem;	/* memory allocated for SS matrix */
+double **xymat;	/* SS matrix for the nvar = nx + ny x- and y-variables */
+int more;	/* for stepping through parts of the dataset */
+char *covset;   /* name of dataset for covariance matrix */
+int param1n, param2n, respn, covn;     /* indices to covset variables */
+int paramlen1, paramlen;        /* length and maximum length of parameter name */
+char paramstr[14];      /* string for declaring variable name variables */
+char *partstr;  /* for declaring part variables in covset */
+int *partv;    /* indexes of part variables for covset */
 
 if (!fname)
 	{
@@ -316,36 +387,70 @@ varv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 xvarv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 markv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 xmarkv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
+partv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 regname = dap_malloc(strlen(fname) + 5, "");
 dap_suffix(regname, fname, ".reg");
 inset(fname);
-dap_vd("_intercept_ -1", 0);
-nx0 = dap_list("_intercept_", varv, dap_maxvar);
-nx0 += dap_list(x0list, varv + 1, dap_maxvar);
-nx1 = dap_list(x1list, varv + nx0, dap_maxvar);
+dap_vd("_intercept_ -1", 0);	/* allocate _intercept_ variable */
+nx0 = dap_list("_intercept_", varv, dap_maxvar);	/* and put in varv */
+nx0 += dap_list(x0list, varv + 1, dap_maxvar);	/* now put in x0 vars */
+nx1 = dap_list(x1list, varv + nx0, dap_maxvar);	/* now put in x1 vars */
 nx = nx0 + nx1;
-ny = dap_list(ylist, varv + nx, dap_maxvar);
+ny = dap_list(ylist, varv + nx, dap_maxvar);	/* now put in y vars */
 nvar = nx + ny;
+nmark = dap_list(marks, markv, dap_maxvar);	/* get mark variable indexes */
+/* do some bookkeeping for covariance dataset */
+covset = dap_malloc(strlen(fname) + 5, "");
+strcpy(covset, fname);
+strcat(covset, ".cov");
+/* find maximum length of variable names */
+for (v = 1, paramlen = strlen("_intercept_"); v < nvar; v++)
+        {
+	paramlen1 = strlen(dap_obs[dap_ono].do_nam[varv[v]]);
+        if (paramlen1 > paramlen)
+                paramlen = paramlen1;
+        }
+/* now set up variables for covariance dataset */
+dap_ono = 2;
+dap_clearobs((char *) NULL);    /* set up dap_obs, make _type_ variable */
+/* this will bomb if a variable name is more than 99 characters long */
+sprintf(paramstr, "_response_ %d", paramlen);
+respn = dap_vd(paramstr, 0);
+sprintf(paramstr, "_param1_ %d", paramlen);
+param1n = dap_vd(paramstr, 0);
+sprintf(paramstr, "_param2_ %d", paramlen);
+param2n = dap_vd(paramstr, 0);
+covn = dap_vd("_cov_ -1", 0);
+partstr = dap_malloc(strlen(marks) + 1, "");
+for (v = 0; v < nmark; v++)
+        {
+        strcpy(partstr, dap_obs[0].do_nam[markv[v]]);
+        sprintf(partstr + strlen(partstr), " %d", dap_obs[0].do_len[markv[v]]);
+        partv[v] = dap_vd(partstr, 1);
+        }
+outset(covset, "");
+/* now back to fname */
+dap_ono = 0;
+/* allocate memory for SS matrix and set up pointers for matrix entries */
 xymem = (double *) dap_malloc(sizeof(double) * nvar * nvar, "");
 xymat = (double **) dap_malloc(sizeof(double *) * nvar, "");
 for (v = 0; v < nvar; v++)
 	xymat[v] = xymem + v * nvar;
-nmark = dap_list(marks, markv, dap_maxvar);
-dap_ono = 1;
-if (xname && xname[0])
+dap_ono = 1;		/* now going to work with dataset xname of x-values */
+if (xname && xname[0])	/* if xname is specified */
 	{
 	inset(xname);
-	nxx = dap_list(x0list, xvarv, dap_maxvar);
+	nxx = dap_list(x0list, xvarv, dap_maxvar);	/* get indexes of x-vars in xname */
 	nxx += dap_list(x1list, xvarv + nxx, dap_maxvar);
-	if (nxx != nx - 1)
+	if (nxx != nx - 1)	/* this had better match, except for _intercept_ */
 		{
 		fprintf(dap_err,
 		"(linreg) %s and %s have different numbers (%d and %d) of x-variables.\n",
 			fname, xname, nx - 1, nxx);
 		exit(1);
 		}
-	if (nmark)
-		{
+	if (nmark)	/* if the datasets are partitioned */
+		{	/* check that fname and xname have same mark vars */
 		for (v = 0; v < nmark; v++)
 			{
 			if ((xmarkv[v] = dap_varnum(dap_obs[0].do_nam[markv[v]])) < 0)
@@ -363,13 +468,13 @@ if (xname && xname[0])
 		exit(1);
 		}
 	}
-else
+else		/* indicate lack of xname dataset */
 	{
 	xvarv[0] = -1;
 	xmarkv[0] = -1;
 	}
-dap_ono = 0;
-for (w = 0; w < nvar; w++)
+dap_ono = 0;	/* now back to dataset fname of observations */
+for (w = 0; w < nvar; w++)	/* check type of variables */
 	{
 	if (dap_obs[0].do_len[varv[w]] != DBL)
 		{
@@ -377,10 +482,11 @@ for (w = 0; w < nvar; w++)
 					dap_obs[0].do_nam[varv[w]]);
 		exit(1);
 		}
-	for (v = 0; v < nvar; v++)
+	for (v = 0; v < nvar; v++)	/* and zero out entire xymat */
 		xymat[v][w] = 0.0;
 	}
-outset(regname, "");
+outset(regname, "");	/* prepare writing of OBS,PRED,LOWER and UPPER values */
+/* now we read in the data and create the SS matrix */
 for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 	{
 	more = step();
@@ -389,8 +495,8 @@ for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 		dap_swap();
 		dap_head(markv, nmark);
 		dap_swap();
-		linreg1(xymat, varv, nx0, nx, ny, nobs,
-				xvarv, markv, xmarkv, nmark, level);
+		linreg1(xymat, varv, nx0, nx, ny, nobs, xvarv, markv, xmarkv, nmark, level,
+				respn, param1n, param2n, covn, partv);
 		for (w = 0; w < nvar; w++)
 			{
 			for (v = 0; v < nvar; v++)
@@ -398,8 +504,15 @@ for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 			}
 		nobs = 0;
 		}
+	/* use standard updating algorithm for computing the SS's on the run for accuracy */
+	/* xymat: 0 row contains sum of values for x and y vars except for _intercept_
+	 * (1 to nx-1) x (1 to nx-1) contains SS for x-variables;
+	 * (1 to nx-1) x (nx to nvar) contains SS for x-vars with each y-var
+	 * (nx to nvar-1) x (1 to nx-1) contains SS for x-vars with each y-var
+	 * (nx, nx) to (nvar-1,nvar-1) diagonal contains SS for each y-var
+	 */
 	if (nobs)
-		{
+		{	/* we do this starting on the second observation */
 		dnobs = (double) nobs;
 		for (v = 1; v < nvar; v++)
 			{
@@ -415,16 +528,19 @@ for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 				}
 			}
 		}
-	for (w = 1; w < nvar; w++)
+	for (w = 1; w < nvar; w++)	/* zero row is sum of values except for _intercept_ */
 		xymat[0][w] += dap_obs[0].do_dbl[varv[w]];
 	}
-dap_free(regname);
-dap_free(varv);
-dap_free(xvarv);
-dap_free(markv);
-dap_free(xmarkv);
-dap_free(xymem);
-dap_free(xymat);
+dap_free(regname, "");
+dap_free(varv, "");
+dap_free(xvarv, "");
+dap_free(markv, "");
+dap_free(xmarkv, "");
+dap_free(xymem, "");
+dap_free(xymat, "");
+dap_free(covset, "");
+dap_free(partstr, "");
+dap_free(partv, "");
 }
 
 void dap_parsey(char *yspec, int *varv)
@@ -509,7 +625,7 @@ else
 	fprintf(dap_err, "(parsey) Expected / in yspec at: %s\n", yspec + l);
 	exit(1);
 	}
-dap_free(vname);
+dap_free(vname, "");
 }
 
 static double vlen(double *v, int nv)
@@ -603,8 +719,8 @@ for (c = nrc - 1; c >= 0; --c)
 for (r = 0; r < nrc; r++)
 	for (c = 0; c < nrc; c++)
 		a[r][c] = inv[r][c];
-dap_free(invmem);
-dap_free(inv);
+dap_free(invmem, "");
+dap_free(inv, "");
 return 1;
 }
 
@@ -726,9 +842,9 @@ do	{
 	} while (++niter <= dap_maxiter && vdiff(beta1, beta0, nx) > dap_ctol * vlen(beta0, nx));
 if (niter > dap_maxiter)
 	fprintf(dap_lst, "Failed to converge after %d iterations.\n", dap_maxiter);
-dap_free(beta1);
-dap_free(v);
-dap_free(step);
+dap_free(beta1, "");
+dap_free(v, "");
+dap_free(step, "");
 return loglike0;
 }
 
@@ -744,7 +860,11 @@ void logreg1(
 	int *markv,	/* array of indices of partitioning variables in main dataset */
 	int *xmarkv,	/* array of indices of partitioning variables in xname dataset */
 	int nmark,	/* number of partitioning variables */
-	double level	/* confidence level for probabilities */
+	double level,	/* confidence level for probabilities */
+	int param1n,	/* for covariance dataset, index of parameter for "row" label */
+	int param2n,	/* for covariance dataset, index of parameter for "column" label */
+	int covn,	/* for covariance dataset, index of parameter for numerical value */
+	int partv[]	/* array of indexes of partioning variables in covset */
 	)
 {
 int typen;			/* index of _type_ variable */
@@ -760,6 +880,7 @@ double xi, xj;
 double logit;
 double selogit;
 int ntrials;
+int v;
 
 /* allocate memory for covariance matrix */
 covmem = (double *) dap_malloc(sizeof(double) * dap_maxvar * dap_maxvar, "");
@@ -803,8 +924,20 @@ loglike0 = irls(x, y, pr, beta, cov, nx0, nobs);
 /* run iterative reweighted least squares on full model */
 loglike1 = irls(x, y, pr, beta, cov, nx, nobs);
 /* don't need probability array any more */
-dap_free(pr);
-/* print results */
+dap_free(pr, "");
+/* now print results and output estimates and SEs, later output the covariance matrix */
+dap_ono = 2;
+/* set up partvars for covariance matrix dataset */
+for (v = 0; v < nmark; v++)
+        {
+        if (dap_obs[0].do_len[markv[v]] == DBL)
+                dap_obs[dap_ono].do_dbl[partv[v]] = dap_obs[0].do_dbl[markv[v]];
+        else if (dap_obs[0].do_len[markv[v]] == INT)
+                dap_obs[dap_ono].do_int[partv[v]] = dap_obs[0].do_int[markv[v]];
+        else
+                strcpy(dap_obs[dap_ono].do_str[partv[v]],
+                        dap_obs[0].do_str[markv[v]]);
+        }
 fprintf(dap_lst, "-2 (Lred - Lfull) = %.6g = ChiSq0[%d]\n",
 		tmp = -2.0 * (loglike0 - loglike1), nx - nx0);
 fprintf(dap_lst, "Prob[ChiSq > ChiSq0] = %.5f\n\n",
@@ -820,7 +953,28 @@ for (i = 0; i < nx; i++)
 		dap_obs[0].do_nam[varv[i]],
 		beta[i], tmp, tmp2,
 		0.00001 * ceil(100000.0 * probchisq(fabs(tmp2), 1)));
+	strcpy(dap_obs[dap_ono].do_str[typen], "ESTIMATE");
+	strcpy(dap_obs[dap_ono].do_str[param2n], dap_obs[0].do_nam[varv[i]]);
+	strcpy(dap_obs[dap_ono].do_str[param1n], "");
+	dap_obs[dap_ono].do_dbl[covn] = beta[i];
+	output();
 	}
+/* now we write out the covariance matrix */
+strcpy(dap_obs[dap_ono].do_str[typen], "COVAR");
+/* "rows" are indexed by i */
+for (i = 0; i < nx; i++)
+	{
+	strcpy(dap_obs[dap_ono].do_str[param1n], dap_obs[0].do_nam[varv[i]]);
+	/* "columns" are indexed by j */
+	for (j = 0; j < nx; j++)
+		{
+		strcpy(dap_obs[dap_ono].do_str[param2n], dap_obs[0].do_nam[varv[j]]);
+		dap_obs[dap_ono].do_dbl[covn] = cov[i][j];
+		output();
+		}
+	}
+/* now back to .lgr output */
+dap_ono = 0;
 if (fabs(level) < 1.0)
 	npt = -zpoint(0.5 * (1.0 - level));
 else
@@ -876,9 +1030,9 @@ dap_ono = 0;
 if (xvarv[0] >= 0)
 	dap_swap();
 /* free allocated memory */
-dap_free(covmem);
-dap_free(cov);
-dap_free(beta);
+dap_free(covmem, "");
+dap_free(cov, "");
+dap_free(beta, "");
 }
 
 /* Run logistic regression on a dataset */
@@ -910,6 +1064,12 @@ double *y[2];		/* the array of corresponding y-values */
 int v;			/* index to array of variable indices */
 int nobs;		/* number of observations */
 int more;		/* more data to be read from dataset? (for partitioning) */
+char *covset;		/* name of dataset for covariance matrix */
+int param1n, param2n, covn;	/* indices to covset variables */
+int paramlen1, paramlen;	/* length and maximum length of parameter name */
+char paramstr[14];	/* string for declaring variable name variables */
+char *partstr;		/* for declaring part variables in covset */
+int *partv;		/* indexes of part variables for covset */
 
 if (!fname)
 	{
@@ -921,6 +1081,7 @@ varv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 xvarv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 markv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 xmarkv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
+partv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 /* allocate string for name of output dataset, 5 for ".reg" with null */
 regname = dap_malloc(strlen(fname) + 5, "");
 /* and assign name */
@@ -950,6 +1111,34 @@ y[0] = ymem;
 y[1] = ymem + dap_maxval;
 /* set up partitioning for main dataset */
 nmark = dap_list(marks, markv, dap_maxvar);
+/* do some bookkeeping for covariance dataset */
+covset = dap_malloc(strlen(fname) + 5, "");
+strcpy(covset, fname);
+strcat(covset, ".cov");
+/* find maximum length of variable names */
+for (v = 1, paramlen = strlen("_intercept_"); v < nx; v++)
+        {
+        paramlen1 = strlen(dap_obs[dap_ono].do_nam[varv[v]]);
+        if (paramlen1 > paramlen)
+                paramlen = paramlen1;
+        }
+/* now set up variables for covariance dataset */
+dap_ono = 2;
+dap_clearobs((char *) NULL);    /* set up dap_obs, make _type_ variable */
+/* this will bomb if a variable name is more than 99 characters long */
+sprintf(paramstr, "_param1_ %d", paramlen);
+param1n = dap_vd(paramstr, 0);
+sprintf(paramstr, "_param2_ %d", paramlen);
+param2n = dap_vd(paramstr, 0);
+covn = dap_vd("_cov_ -1", 0);
+partstr = dap_malloc(strlen(marks) + 1, "");
+for (v = 0; v < nmark; v++)
+        {
+        strcpy(partstr, dap_obs[0].do_nam[markv[v]]);
+        sprintf(partstr + strlen(partstr), " %d", dap_obs[0].do_len[markv[v]]);
+        partv[v] = dap_vd(partstr, 1);
+        }
+outset(covset, "");
 /* now switch to xname dataset */
 dap_ono = 1;
 if (xname && xname[0])	/* if caller wants predicted values */
@@ -1004,8 +1193,8 @@ for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 		dap_head(markv, nmark);
 		dap_swap();
 		/* and run logistic regression on it */
-		logreg1(y, x, nx0, nx, nobs,
-				varv, xvarv, markv, xmarkv, nmark, level);
+		logreg1(y, x, nx0, nx, nobs, varv, xvarv, markv, xmarkv, nmark, level,
+					param1n, param2n, covn, partv);
 		nobs = 0;	/* clean slate for next part */
 		}
 	if (nobs < dap_maxval)	/* if room for more data... */
@@ -1026,14 +1215,17 @@ for (nobs = 0, dap_mark(), more = 1; more; nobs++)
 		}
 	}
 /* release all allocated memory */
-dap_free(regname);
-dap_free(varv);
-dap_free(xvarv);
-dap_free(markv);
-dap_free(xmarkv);
-dap_free(xmem);
-dap_free(x);
-dap_free(ymem);
+dap_free(regname, "");
+dap_free(varv, "");
+dap_free(xvarv, "");
+dap_free(markv, "");
+dap_free(xmarkv, "");
+dap_free(xmem, "");
+dap_free(x, "");
+dap_free(ymem, "");
+dap_free(covset, "");
+dap_free(partstr, "");
+dap_free(partv, "");
 }
 
 typedef struct {
@@ -1333,9 +1525,9 @@ else
 						nval - 1);
 		}
 	}
-dap_free(levnobs);
-dap_free(rank1);
-dap_free(sumr);
+dap_free(levnobs, "");
+dap_free(rank1, "");
+dap_free(sumr, "");
 dap_swap();
 }
 
@@ -1448,8 +1640,8 @@ for (more = 1, nlevels = 0, nval = 0, nobs = 0; more; nobs++)
 			}
 		}
 	}
-dap_free(val);
-dap_free(markv);
-dap_free(levmem);
-dap_free(level);
+dap_free(val, "");
+dap_free(markv, "");
+dap_free(levmem, "");
+dap_free(level, "");
 }
