@@ -21,6 +21,7 @@
 #include "externs.h"
 
 extern dataobs dap_obs[];
+extern int dap_ono;
 extern FILE *dap_lst;
 extern FILE *dap_err;
 extern char *dap_dapname;
@@ -87,12 +88,15 @@ return c;
 
 static void categ1(double **tab, int ncell, int *varv, int nvar,
 		double (*expect)(), double *param,
-		char *select, char *selcodes, char *trace)
+		char *select, char *selcodes,
+		int param1n, int param2n, int covn,
+		int partv[], int partv2[], int npart, char *trace)
 {
+int typen;	/* index of _type_ variable */
 int sparam;	/* number of selected parameters */
 int sparamr;	/* number of selected parameters in reduced model */
 int nparam;	/* number of parameters */
-int c;
+int c, c1, c0;	/* indexes to selection string */
 double *x;
 double *xch;
 int p;
@@ -102,21 +106,21 @@ int v;
 double likerat;
 double likered;
 double pearson;
-int typen;
-double *infomem;
-double **info;
+double *infomem;	/* memory for information matrix */
+double **info;		/* information matrix */
 int p1, p2;
 double lpp, lpm, lmp, lmm;
 double h, halfh;
-int s;
+int s, s1;	/* indexes to info and covariance matrices */
+int nonsing;	/* info matrix non-singular? */
 
-sparamr = 0;
-likered = 0.0;
 if ((typen = dap_varnum("_type_")) < 0)
 	{
 	fputs("(categ1) missing _type_ variable\n", dap_err);
 	exit(1);
 	}
+sparamr = 0;
+likered = 0.0;
 nc = ncell;
 ex = expect;
 x = (double *) dap_malloc(sizeof(double) * strlen(sel), "");
@@ -151,11 +155,8 @@ if (selred)
 	tol = dap_cattol * step;
 	dap_maximize(&loglike, sparamr, x, step, tol, trace);
 	for (c = 0, likerat = 0.0; c < ncell; c++)
-		{
 		likered += (tab[c][0] + dap_addtozero) *
 			log((tab[c][0] + dap_addtozero) / expect(allparam, tab[c] + 1));
-		output();
-		}
 	likered *= 2.0;
 	}
 sel = selcodes;
@@ -234,11 +235,25 @@ for (p1 = 0; p1 < sparam; p1++)
 		info[p1][p2] = -(lpp - lpm - lmp + lmm) / (h * h);
 		info[p2][p1] = info[p1][p2];
 		}
-if (!dap_invert(info, sparam))
-	{
+if (!(nonsing = dap_invert(info, sparam)))
 	fputs("(categ1) covariance matrix is singular\n", dap_err);
+/* now print and output estimates and SEs, later output the covariance matrix */
+dap_ono = 1;
+if ((typen = dap_varnum("_type_")) < 0)
+	{
+	fprintf(dap_err, "(categ1) output dataset has no _type_ variable\n");
 	exit(1);
 	}
+for (v = 0; v < npart; v++)
+	{
+	if (dap_obs[0].do_len[partv[v]] == DBL)
+		dap_obs[dap_ono].do_dbl[partv2[v]] = dap_obs[0].do_dbl[partv[v]];
+	else if (dap_obs[0].do_len[partv[v]] == INT)
+		dap_obs[dap_ono].do_int[partv2[v]] = dap_obs[0].do_int[partv[v]];
+	else
+		strcpy(dap_obs[dap_ono].do_str[partv2[v]],
+			dap_obs[0].do_str[partv[v]]);
+	}
 fputs("Maximum likelihood estimation\n\n", dap_lst);
 fprintf(dap_lst, "Cell count: %s\n", dap_obs[0].do_nam[varv[0]]);
 fputs("Class and aux variables:", dap_lst);
@@ -248,7 +263,9 @@ putc('\n', dap_lst);
 fputs("\nStatistic              df      Prob\n", dap_lst);
 fprintf(dap_lst, "G2[Model]   = %6.2f  %3d    %.4f\n",
 	likerat, ncell - sparam,
-	ceil(10000.0 * probchisq(likerat, ncell - sparam)) / 10000.0);
+	((ncell > sparam) ?
+		(ceil(10000.0 * probchisq(likerat, ncell - sparam)) / 10000.0) :
+		1.0));
 if (selred)
 	{
 	fprintf(dap_lst, "G2[Reduced] = %6.2f  %3d    %.4f\n",
@@ -260,7 +277,9 @@ if (selred)
 	}
 fprintf(dap_lst, "X2[Model]   = %6.2f  %3d    %.4f\n",
 	pearson, ncell - sparam,
-	ceil(10000.0 * probchisq(pearson, ncell - sparam)) / 10000.0);
+	((ncell > sparam) ?
+		(ceil(10000.0 * probchisq(pearson, ncell - sparam)) / 10000.0) :
+		1.0));
 putc('\n', dap_lst);
 fputs("    Estimate          ASE  Model  Parameter\n", dap_lst);
 for (c = 0; select[c] == ' '; c++)
@@ -272,8 +291,10 @@ for (p = 0, s = 0; p < nparam; p++)
 		fputs("              ", dap_lst);
 	else
 		{
-		fprintf(dap_lst, "%12g  ", sqrt(info[s][s]));
-		s++;
+		if (nonsing)
+			fprintf(dap_lst, "%12g  ", sqrt(info[s][s]));
+		else
+			fputs("           ?  ", dap_lst);
 		}
 	switch (selcodes[p])
 		{
@@ -291,28 +312,96 @@ for (p = 0, s = 0; p < nparam; p++)
 			;
 		break;
 		}
-	while (select[c] && select[c] != ' ')
+	for (p2 = 0; select[c] && select[c] != ' '; c++, p2++)
 		{
 		putc(select[c], dap_lst);
+		dap_obs[dap_ono].do_str[param2n][p2] = select[c];
+		}
+	dap_obs[dap_ono].do_str[param2n][p2] = '\0';
+	while (select[c] == ' ')
 		c++;
+	if (sel[p] != '!')
+		{
+		strcpy(dap_obs[dap_ono].do_str[typen], "ESTIMATE");
+		strcpy(dap_obs[dap_ono].do_str[param1n], "");
+		dap_obs[dap_ono].do_dbl[covn] = allparam[p];
+		output();
+		s++;
+		}
+	putc('\n', dap_lst);
+	}
+putc('\n', dap_lst);
+/* now output covariance matrix into dataset */
+strcpy(dap_obs[dap_ono].do_str[typen], "COVAR");
+for (c = 0; select[c] == ' '; c++)
+	;
+for (p = 0, s = 0; p < nparam; p++)
+	{
+	if (selcodes[p] != '1')
+		{
+		for (++c; select[c] == ' '; c++)
+			;
 		}
 	while (select[c] == ' ')
 		c++;
-	putc('\n', dap_lst);
+	for (c1 = 0; select[c1] == ' '; c1++)
+		;
+	for (p1 = 0, s1 = 0; p1 < nparam; p1++)
+		{
+		if (sel[p] != '!' && sel[p1] != '!')
+			dap_obs[dap_ono].do_dbl[covn] = info[s][s1];
+		if (sel[p1] != '!')
+			s1++;
+		if (selcodes[p1] != '1')
+			{
+			for (++c1; select[c1] == ' '; c1++)
+				;
+			}
+		if (sel[p] != '!' && sel[p1] != '!')
+			{
+			for (p2 = 0; select[c1] && select[c1] != ' '; c1++, p2++)
+				dap_obs[dap_ono].do_str[param2n][p2] = select[c1];
+			dap_obs[dap_ono].do_str[param2n][p2] = '\0';
+			c0 = c;
+			if (select[c0] == '?')
+				{
+				for (c0++; select[c0] == ' '; c0++)
+					;
+				}
+			for (p2 = 0; select[c0] && select[c0] != ' '; c0++, p2++)
+				dap_obs[dap_ono].do_str[param1n][p2] = select[c0];
+			dap_obs[dap_ono].do_str[param1n][p2] = '\0';
+			output();
+			}
+		while (select[c1] == ' ')
+			c1++;
+		}
+	while (select[c] && select[c] != ' ')
+		c++;
+	if (sel[p] != '!')
+		s++;
 	}
-dap_free(x);
-dap_free(xch);
-dap_free(infomem);
-dap_free(info);
+dap_ono = 0;
+dap_free(x, "");
+dap_free(xch, "");
+dap_free(infomem, "");
+dap_free(info, "");
 }
 
-void categ(char *dataset, char *varlist, double (*expect)(),
+void categ(char *dname, char *varlist, double (*expect)(),
 		double *param, char *select, char *part, char *trace)
 {
 int p;
-char *catset;
+char *filset;	/* name of dataset with no missing cells */
+char *filarg;	/* argument for dataset for FILL */
+char *catset;	/* name of dataset for tables of observed and fitted values */
+char *covset;	/* name of dataset for covariance matrix */
+int param1n, param2n, covn;	/* indices to covset variables */
+int paramlen1, paramlen;	/* length and maximum length of parameter name */
+char paramstr[12];	/* string for declaring parameter name variables */
+char *partstr;	/* for declaring part variables in covset */
 int *varv;
-int *partv;
+int *partv, *partv2;	/* indexes of part variables for catset and covset */
 int nvar;
 int npart;
 int more;
@@ -325,11 +414,39 @@ int s;
 
 varv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
 partv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
-catset = dap_malloc(strlen(dataset) + 5, "");
-strcpy(catset, dataset);
+partv2 = (int *) dap_malloc(sizeof(int) * dap_maxvar, "dap_maxvar");
+filarg = dap_malloc(strlen(varlist) + strlen(part) + 8, "");
+strcpy(filarg, "FILL ");
+for (s = 0; varlist[s] == ' '; s++)
+	;	/* skip leading blanks before count arg */
+for (v = 5; varlist[s] && varlist[s] != ' '; )
+	filarg[v++] = varlist[s++];	/* copy in count arg */
+filarg[v++] = ':';
+strcpy(filarg + v, part);	/* part vars need to come first */
+strcat(filarg, " ");
+strcat(filarg, varlist + s);
+filset = dap_malloc(strlen(dname) + 5, "");
+strcpy(filset, dname);
+strcat(filset, ".fil");
+dataset(dname, filset, filarg);
+catset = dap_malloc(strlen(dname) + 5, "");
+strcpy(catset, dname);
 strcat(catset, ".cat");
-inset(dataset);
-outset(catset, "");
+covset = dap_malloc(strlen(dname) + 5, "");
+strcpy(covset, dname);
+strcat(covset, ".cov");
+/* find maximum length of parameter names */
+for (s = 0, paramlen = 0; select[s]; )
+	{
+	while (select[s] == ' ' || select[s] == '?' || select[s] == '!')
+		s++;
+	for (paramlen1 = 0; select[s] && select[s] != ' '; s++)
+		paramlen1++;
+	if (paramlen1 > paramlen)
+		paramlen = paramlen1;
+	}
+dap_ono = 0;
+inset(filset);
 nvar = dap_list(varlist, varv, dap_maxvar);
 for (v = 1; v < nvar; v++)
 	{
@@ -341,6 +458,26 @@ fprintf(dap_err, "(categ) classification or auxiliary variable not of type doubl
 		}
 	}
 npart = dap_list(part, partv, dap_maxvar);
+/* now set up variables for covariance dataset */
+dap_ono = 1;
+dap_clearobs((char *) NULL);	/* set up dap_obs, make _type_ variable */
+/* this will bomb if a parameter name is more than 99 characters long */
+sprintf(paramstr, "_param1_ %d", paramlen);
+param1n = dap_vd(paramstr, 0);
+sprintf(paramstr, "_param2_ %d", paramlen);
+param2n = dap_vd(paramstr, 0);
+covn = dap_vd("_cov_ -1", 0);
+partstr = dap_malloc(strlen(part) + 1, "");
+for (v = 0; v < npart; v++)
+	{
+	strcpy(partstr, dap_obs[0].do_nam[partv[v]]);
+	sprintf(partstr + strlen(partstr), " %d", dap_obs[0].do_len[partv[v]]);
+	partv2[v] = dap_vd(partstr, 1);
+	}
+outset(covset, "");
+/* now back to catset */
+dap_ono = 0;
+outset(catset, "");
 tabmem = (double *) dap_malloc(sizeof(double) * nvar * dap_maxcell, "");
 tab = (double **) dap_malloc (sizeof(double *) * dap_maxcell, "");
 for (ncell = 0; ncell < dap_maxcell; ncell++)
@@ -348,8 +485,6 @@ for (ncell = 0; ncell < dap_maxcell; ncell++)
 selcodes = dap_malloc(strlen(select) + 1, "");
 nparam = selparse(select, selcodes);
 allparam = (double *) dap_malloc(sizeof(double) * nparam, "");
-for (p = 0; p < nparam; p++)
-	allparam[p] = param[p];
 sel = selcodes;
 if (index(selcodes, '?'))
 	{
@@ -365,6 +500,8 @@ if (index(selcodes, '?'))
 	}
 else
 	selred = NULL;
+for (p = 0; p < nparam; p++)
+	allparam[p] = param[p];
 for (ncell = 0, more = 1; more; )
 	{
 	more = step();
@@ -372,35 +509,45 @@ for (ncell = 0, more = 1; more; )
 		{
 		dap_swap();
 		dap_head(partv, npart);
-		categ1(tab, ncell, varv, nvar, expect, param,
-						select, selcodes, trace);
+		categ1(tab, ncell, varv, nvar, expect, param, select, selcodes,
+				param1n, param2n, covn, partv, partv2, npart, trace);
 		dap_swap();
 		ncell = 0;
+		for (p = 0; p < nparam; p++)
+			allparam[p] = param[p];
 		}
-	if (ncell < dap_maxcell)
+	if (more)
 		{
-		for (v = 0; v < nvar; v++)
-			tab[ncell][v] = dap_obs[0].do_dbl[varv[v]];
-		ncell++;
-		}
-	else
-		{
-		fputs("(categ) too many cells\n", dap_err);
-		exit(1);
+		if (ncell < dap_maxcell)
+			{
+			for (v = 0; v < nvar; v++)
+				tab[ncell][v] = dap_obs[0].do_dbl[varv[v]];
+			ncell++;
+			}
+		else
+			{
+			fputs("(categ) too many cells\n", dap_err);
+			exit(1);
+			}
 		}
 	}
 if (selred)
 	{
-	dap_free(selred);
+	dap_free(selred, "");
 	selred = NULL;
 	}
-dap_free(varv);
-dap_free(partv);
-dap_free(catset);
-dap_free(tabmem);
-dap_free(tab);
-dap_free(allparam);
-dap_free(selcodes);
+dap_free(filarg, "");
+dap_free(varv, "");
+dap_free(partv, "");
+dap_free(partv2, "");
+dap_free(filset, "");
+dap_free(catset, "");
+dap_free(covset, "");
+dap_free(partstr, "");
+dap_free(tabmem, "");
+dap_free(tab, "");
+dap_free(allparam, "");
+dap_free(selcodes, "");
 }
 
 /* compute expected cell counts from the parameters and the
@@ -595,7 +742,7 @@ for (t1 = 0; t1 <= nterm; t1++)
 			}
 		}
 	}
-dap_free(oneclass);
+dap_free(oneclass, "");
 return nterm;
 }
 
@@ -612,6 +759,8 @@ struct node *next;
  */
 void loglin(char *fname, char *varlist, char *model0, char *model1, char *part)
 {
+char *fnamefil;		/* copy of fname with empty cells filled */
+char *filarg;		/* variable names for FILL */
 char *fname1;		/* copy of fname to use for categ, which requires doubles
 			 * instead of strings for classification variables
 			 */
@@ -664,6 +813,20 @@ int onelen;		/* length of one class value */
 char formstr[8];	/* "  %-xxs" + '\0' */
 valnode *nodeptr;	/* for stepping through node list */
 
+fnamefil = dap_malloc(strlen(fname) + 5, "");
+strcpy(fnamefil, fname);
+strcat(fnamefil, ".fil");
+filarg = dap_malloc(strlen(varlist) + strlen(part) + 8, "");
+strcpy(filarg, "FILL ");
+for (s = 0; varlist[s] == ' '; s++)
+	;	/* skip leading blanks before count arg */
+for (l = 5; varlist[s] && varlist[s] != ' '; )
+	filarg[l++] = varlist[s++];	/* copy in count arg */
+filarg[l++] = ':';
+strcpy(filarg + l, part);	/* part vars need to come first */
+strcat(filarg, " ");
+strcat(filarg, varlist + s);
+dataset(fname, fnamefil, filarg);
 fname1 = dap_malloc(strlen(fname) + 5, "");
 strcpy(fname1, fname);
 strcat(fname1, ".llm");
@@ -701,7 +864,7 @@ endcv = (valnode **) dap_malloc(sizeof(valnode *) * nclass, "");
 /* and maxlen[c] = max length of value for class var c */
 maxlen = (int *) dap_malloc(sizeof(int) * nclass, "");
 
-inset(fname);
+inset(fnamefil);
 
 /* part vars go in outlist, not in varlist1 */
 npart = dap_list(part, classv, dap_maxvar);
@@ -1090,36 +1253,345 @@ while (step())
 	output();
 	}
 
-dap_free(fname1);
-dap_free(catname);
-dap_free(varlist1);
-dap_free(outlist);
-dap_free(vardef);
-dap_free(classv);
-dap_free(maxval);
-dap_free(maxval1);
-dap_free(term);
-dap_free(param);
-dap_free(select);
-dap_free(selterm);
-dap_free(sub);
-dap_free(classmem);
-dap_free(class);
-dap_free(pattern);
-dap_free(paramlistmem);
-dap_free(paramlist);
+dap_free(fnamefil, "");
+dap_free(filarg, "");
+dap_free(fname1, "");
+dap_free(catname, "");
+dap_free(varlist1, "");
+dap_free(outlist, "");
+dap_free(vardef, "");
+dap_free(classv, "");
+dap_free(maxval, "");
+dap_free(maxval1, "");
+dap_free(term, "");
+dap_free(param, "");
+dap_free(select, "");
+dap_free(selterm, "");
+dap_free(sub, "");
+dap_free(classmem, "");
+dap_free(class, "");
+dap_free(pattern, "");
+dap_free(paramlistmem, "");
+dap_free(paramlist, "");
 for (c = 0; c < nclass; c++)
 	{
 	while (classval[c])
 		{
-		dap_free(classval[c]->value);
+		dap_free(classval[c]->value, "");
 		endcv[c] = classval[c]->next;
-		dap_free(classval[c]);
+		dap_free(classval[c], "");
 		classval[c] = endcv[c];
 		}
 	}
-dap_free(classval);
-dap_free(endcv);
-dap_free(maxlen);
+dap_free(classval, "");
+dap_free(endcv, "");
+dap_free(maxlen, "");
+}
 
+/* estimate: use a estimate and covariance matrix dataset such as that
+ * produced by categ() to define and test parameters
+ */
+
+static int findparam(char *pname, char *param[], int nparam)
+{
+int p;
+
+for (p = 0; p < nparam; p++)
+	{
+	if (!strcmp(pname, param[p]))
+		return p;
+	}
+return -1;
+}
+
+void estimate(char *fname, char *parameters, char *definitions, char *part)
+{
+char *parammem;		/* memory to hold parameter names */
+char **param;		/* pointers to parameter names */
+int start, end;	/* for stepping through 'definitions' */
+int nparam;		/* number of parameters */
+int p, p1;		/* index to parameter arrays */
+int defstate;		/* 0 = new def, 1 = got '=' */
+double *defmem;		/* memory for def */
+double **def;		/* definition for each parameter: each entry contains the
+                         * coefficient for the parameter in the def or 0 if not in the def
+			 */
+double coeff;		/* coefficient of def entry */
+double sign;		/* sign of the coefficient */
+double place;		/* for conversion from string to double */
+int pnum1, pnum2, pnum3;	/* parameter numbers */
+int defnum;		/* number of parameter being defined */
+int ninput;		/* number of parameters to read in */
+double *estimate;	/* vector of estimates */
+double *covmem;		/* memory for covariance matrix */
+double **cov;		/* covariance matrix */
+int typen, param1n, param2n, covn;	/* indices of input variables */
+int more;		/* for checking if there are more data lines to read in */
+int *partv;		/* indices of the partitioning variables */
+int npart;		/* number of partitioning variables */
+
+parammem = dap_malloc(strlen(definitions) + 1, "");
+/* divide by 2 because params are separated; almost always still too many */
+/* this is just (an overestimate of) the max, not the real nparam */
+nparam = (strlen(parameters) + strlen(definitions)) / 2;
+param = (char **) dap_malloc(sizeof(char *) * nparam, "");
+defmem = (double *) dap_malloc(sizeof(double) * nparam * nparam, "");
+def = (double **) dap_malloc(sizeof(double *) * nparam, "");
+for (p = 0; p < nparam; p++)
+	{
+	def[p] = defmem + p * nparam;
+	for (p1 = 0; p1 < nparam; p1++)
+		def[p][p1] = 0.0;	/* clear them all to start */
+	}
+/* get parameter names from 'parameters' */
+for (start = 0; parameters[start] == ' '; start++)
+	;
+for (nparam = 0, p = 0; parameters[start]; start = end)
+	{
+	/* tentatively copy param from 'parameters' to param list */
+	param[nparam++] = parammem + p;
+	for (end = start; parameters[end] && parameters[end] != ' '; )
+		parammem[p++] = parameters[end++];
+	parammem[p++] = '\0';
+	while (parameters[end] == ' ')
+		end++;
+	}
+ninput = nparam;
+/* now get parameter names from 'definitions' */
+for (start = 0; definitions[start] == ' '; start++)
+	;	/* get to first parameter */
+for (defstate = 0; definitions[start]; start = end)
+	{
+	/* tentatively copy param from definition to param list */
+	param[nparam] = parammem + p;
+	if (definitions[start] == '+' || definitions[start] == '-')
+		{
+		sign = 2.0 * ((double) (definitions[start] == '+')) - 1.0;
+		for (start++; definitions[start] == ' '; start++)
+			;
+		}
+	if (definitions[start] == '.' ||
+			('0' <= definitions[start] && definitions[start] <= '9'))
+		{
+		for (coeff = 0.0, place = 0.0;
+				definitions[start] == '.' ||
+				('0' <= definitions[start] && definitions[start] <= '9');
+				start++)
+			{
+			if (definitions[start] == '.')
+				{
+				if (place > 0.0)
+					{
+					fprintf(dap_err,
+					"(estimate) bad coefficient in definition: %s\n",
+						definitions + start);
+					exit(1);
+					}
+				else
+					place = 1.0;
+				}
+			else
+				{
+				coeff = 10.0 * coeff + (double) (definitions[start] - '0');
+				if (place > 0.0)
+					place *= 10.0;
+				}
+			}
+		coeff *= sign;
+		if (place > 0.0)
+			coeff /= place;
+		while (definitions[start] == ' ')
+			start++;
+		}
+	else
+		coeff = sign;
+	for (end = start;
+			definitions[end] && definitions[end] != ' ' &&
+			definitions[end] != '=' && definitions[end] != '+' &&
+			definitions[end] != '-'; )
+		parammem[p++] = definitions[end++];
+	parammem[p++] = '\0';
+	if ((pnum1 = findparam(param[nparam], param, nparam)) < 0)	/* it's new */
+		nparam++;	/* count it */
+	else
+		p = param[nparam] - parammem;	/* reset p */
+	while (definitions[end] == ' ')
+		end++;
+	switch (defstate)
+		{
+	case 0:		/* new definition */
+		if (definitions[end] == '=')
+			{
+			defstate = 1;
+			for (end++; definitions[end] == ' '; end++)
+				;
+			}
+		else
+			{
+			fprintf(dap_err,
+			"(estimate) definition starting at %s missing an =\n",
+					definitions + start);
+			exit(1);
+			}
+		defnum = nparam - 1;
+		sign = 1.0;	/* default for first parameter in definition */
+		break;
+	case 1:
+		if (pnum1 < 0)
+			{
+			fprintf(dap_err,
+			"(estimate) undefined parameter %s in definition\n",
+					param[nparam - 1]);
+			exit(1);
+			}
+		if (sign == 0.0)
+			{
+			fprintf(dap_err,
+		"(estimate) missing sign or coefficient for parameter %s in definition\n",
+					param[nparam - 1]);
+			exit(1);
+			}
+		def[defnum][pnum1] = coeff;
+		if (definitions[end] != '+' && definitions[end] != '-')
+			/* end of definition */
+			defstate = 0;
+		sign = 0.0;
+		break;
+		}
+	}
+estimate = (double *) dap_malloc(sizeof(double) * nparam, "");
+covmem = (double *) dap_malloc(sizeof(double) * nparam * nparam, "");
+cov = (double **) dap_malloc(sizeof(double *) * nparam, "");
+for (p = 0; p < nparam; p++)
+	cov[p] = covmem + nparam * p;
+inset(fname);
+if ((typen = dap_varnum("_type_")) < 0)
+	{
+	fprintf(dap_err, "(estimate) missing _type_ variable in dataset %s\n", fname);
+	exit(1);
+	}
+if ((param1n = dap_varnum("_param1_")) < 0)
+	{
+	fprintf(dap_err, "(estimate) missing _param1_ variable in dataset %s\n", fname);
+	exit(1);
+	}
+if ((param2n = dap_varnum("_param2_")) < 0)
+	{
+	fprintf(dap_err, "(estimate) missing _param2_ variable in dataset %s\n", fname);
+	exit(1);
+	}
+if ((covn = dap_varnum("_cov_")) < 0)
+	{
+	fprintf(dap_err, "(estimate) missing _cov_ variable in dataset %s\n", fname);
+	exit(1);
+	}
+partv = (int *) dap_malloc(sizeof(int) * dap_maxvar, "");
+npart = dap_list(part, partv, dap_maxvar);
+for (pnum1 = 0; pnum1 < nparam; pnum1++)
+	{
+	estimate[pnum1] = 0.0 / 0.0;
+	for (pnum2 = 0; pnum2 < nparam; pnum2++)
+		cov[pnum1][pnum2] = 0.0 / 0.0;
+	}
+for (more = 1; more; )
+	{
+	more = step();
+	if (dap_newpart(partv, npart))
+		{
+		dap_swap();
+		dap_head(partv, npart);
+		fputs("    Estimate           SE  Parameter\n", dap_lst);
+		for (pnum1 = ninput; pnum1 < nparam; pnum1++)
+			{
+			/* compute estimate of defined parameter */
+			estimate[pnum1] = 0.0;
+			for (pnum2 = 0; pnum2 < pnum1; pnum2++)
+				{
+				if (def[pnum1][pnum2] && !finite(estimate[pnum2]))
+					{
+					fprintf(dap_err,
+				"(estimate) estimate for parameter %s not in dataset %s\n",
+						param[pnum2], fname);
+					exit(1);
+					}
+				estimate[pnum1] += def[pnum1][pnum2] * estimate[pnum2];
+				}
+			/* compute variance of defined parameter */
+			cov[pnum1][pnum1] = 0.0;
+			for (pnum2 = 0; pnum2 < pnum1; pnum2++)
+				{
+				for (pnum3 = 0; pnum3 < pnum1; pnum3++)
+					{
+					coeff = def[pnum1][pnum2] * def[pnum1][pnum3];
+					cov[pnum1][pnum1] += coeff * cov[pnum2][pnum3];
+					}
+				}
+			/* compute covariance of def. param. with others */
+			for (pnum2 = 0; pnum2 < pnum1; pnum2++)
+				{
+				cov[pnum1][pnum2] = 0.0;
+				for (pnum3 = 0; pnum3 < pnum1; pnum3++)
+					{
+					coeff = def[pnum1][pnum2];
+					cov[pnum1][pnum2] += coeff * cov[pnum2][pnum3];
+					}
+				cov[pnum2][pnum1] = cov[pnum1][pnum2];
+				}
+			}
+		for (pnum1 = ninput; pnum1 < nparam; pnum1++)
+			{
+			fprintf(dap_lst, "%12g %12g  %s =",
+				estimate[pnum1], sqrt(cov[pnum1][pnum1]), param[pnum1]);
+			for (pnum2 = 0; pnum2 < nparam; pnum2++)
+				{
+				if (def[pnum1][pnum2] != 0.0)
+					{
+					putc(' ', dap_lst);
+					if (def[pnum1][pnum2] > 0.0)
+						putc('+', dap_lst);
+					if (def[pnum1][pnum2] == 1.0)
+						fprintf(dap_lst, "%s", param[pnum2]);
+					else if (def[pnum1][pnum2] == -1.0)
+						fprintf(dap_lst, " -%s", param[pnum2]);
+					else
+						fprintf(dap_lst, "%g%s",
+							def[pnum1][pnum2], param[pnum2]);
+					}
+				}
+			putc('\n', dap_lst);
+			}
+		dap_swap();
+		for (pnum1 = 0; pnum1 < nparam; pnum1++)
+			{
+			estimate[pnum1] = 0.0 / 0.0;
+			for (pnum2 = 0; pnum2 < nparam; pnum2++)
+				cov[pnum1][pnum2] = 0.0 / 0.0;
+			}
+		}
+	if (more)
+		{
+		if (!strcmp(dap_obs[dap_ono].do_str[typen], "ESTIMATE"))
+			{
+			if ((pnum2 = findparam(dap_obs[dap_ono].do_str[param2n],
+						param, ninput)) >= 0)
+				estimate[pnum2] = dap_obs[dap_ono].do_dbl[covn];
+			}
+		else if (!strcmp(dap_obs[dap_ono].do_str[typen], "COVAR"))
+			{
+			if ((pnum1 = findparam(dap_obs[dap_ono].do_str[param1n],
+						param, ninput)) >= 0 &&
+				(pnum2 = findparam(dap_obs[dap_ono].do_str[param2n],
+						param, ninput)) >= 0)
+				cov[pnum1][pnum2] = dap_obs[dap_ono].do_dbl[covn];
+			}
+		}
+	}
+dap_free(parammem, "");
+dap_free(param, "");
+dap_free(defmem, "");
+dap_free(def, "");
+dap_free(estimate, "");
+dap_free(covmem, "");
+dap_free(cov, "");
+dap_free(partv, "");
 }
