@@ -310,8 +310,8 @@ for (f0 = (*f)(x), nsteps = 0; ; nsteps++)
 		vcopy(x, x1, nx);
 		}
 	}
-dap_free(x1);
-dap_free(x2);
-dap_free(dir);
+dap_free(x1, "");
+dap_free(x2, "");
+dap_free(dir, "");
 return f0;
 }
