@@ -314,44 +314,6 @@ else
 #define AXISMARGIN 0.05
 #define RADIUSFACTOR 100.0
 
-/* round x to number of places indicated by ndigs,
- * up or down indicated by updown (= 1 for up, 0 for nearest, -1 for down)
- */
-static double rnd(double x, int ndigs, int updown)
-{
-double tx;
-double pow10;
-double precision;
-
-if (x == 0.0)
-	return 0.0;
-for (precision = 1.0; ndigs > 0; ndigs--)
-	precision *= 10.0;
-tx = x;		/* so we don't have to worry about the sign */
-if (tx < 0.0)
-	tx = -x;
-if (tx >= 1.0)
-	{
-	for (pow10 = 1.0; pow10 < tx; pow10 *= 10.0)
-		; 
-	}
-else
-	{
-	for (pow10 = 1.0; 1.0 / pow10 > tx; pow10 *= 10.0)
-		;
-	pow10 /= 10.0;
-	}
-pow10 = precision / pow10;
-if (pow10 < 1.0)
-	pow10 = 1.0;
-if (updown == 1)
-	return ceil(x * pow10) / pow10;
-else if (updown == -1)
-	return floor(x * pow10) / pow10;
-else
-	return rint(x * pow10) / pow10;
-}
-
 /* make label; OK this can crash the eventual sprintf if you have some really,
  * really big number with more than dap_maxtxt-1 digits, but then your picture
  * wouldn't look so good either.
@@ -708,10 +670,8 @@ else
 	}
 minxt = minx;
 minx -= AXISMARGIN * (maxx - minx);
-minx = rnd(minx, nxdigs, -1);
 maxxt = maxx;
 maxx += AXISMARGIN * (maxx - minx);
-maxx = rnd(maxx, nxdigs, 1);
 if (finite(specymin))
 	miny = specymin;
 else
@@ -729,10 +689,8 @@ else
 minyt = miny;
 if (miny != 0.0)
 	miny -= AXISMARGIN * (maxy - miny);
-miny = rnd(miny, nydigs, -1);
 maxyt = maxy;
 maxy += AXISMARGIN * (maxy - miny);
-maxy = rnd(maxy, nydigs, 1);
 if (specxticks >= 0)
 	nxticks = specxticks;
 else
@@ -851,9 +809,9 @@ if (autopos)
 	pict_translate(p, LEFT + 0.5 * (width - (minx + maxx)),
 			BOTTOM + 0.5 * (height - (miny + maxy)));
 	}
-dap_free(xticks);
-dap_free(yticks);
-dap_free(word);
+dap_free(xticks, "");
+dap_free(yticks, "");
+dap_free(word, "");
 return sqrt((width * width + height * height) / RADIUSFACTOR);
 }
 
@@ -1082,9 +1040,9 @@ for (pn = 0, p = firstp; p; pn++)
 	else
 		p = p->pict_next;
 	}
-dap_free(outname);
-dap_free(pict_font);
-dap_free(pict_txt);
+dap_free(outname, "");
+dap_free(pict_font, "");
+dap_free(pict_txt, "");
 }
 
 pict *pict_rest(char *dataset)
@@ -1337,6 +1295,6 @@ for (pn = 0; pn < npicts; pn++)
 			dap_obs[0].do_dbl[pict_tang], dap_obs[0].do_str[pict_pos]);
 		}
 	}
-dap_free(inname);
+dap_free(inname, "");
 return p;
 }
